//
//  SundialView.m
//  SunDial
//
//  Created by SJSU Students on 4/30/12.
//  Copyright (c) 2012 SJSU Students. All rights reserved.
//

#import "SundialView.h"

#import "SundialData.h"

@interface SundialView()

@property (strong, nonatomic) NSString *theTime;

@property (strong, nonatomic) SundialData *aDial;
@property BOOL drawn;

@property CGSize aSize;
@property CGRect aRect;
@property BOOL tilted;

- (void)getGyro:(SundialData *)theDial;
- (void)checkTilt:(SundialData *)myData;

@end

@implementation SundialView

@synthesize theTime = _theTime;
@synthesize aDial = _aDial;
@synthesize drawn = _drawn;
@synthesize aSize = _aSize;
@synthesize theOrigin = _theOrigin;
@synthesize aRect = _aRect;
@synthesize tilted = _tilted;

- (id)initWithFrame:(CGRect)frame
{
    if (self = [super initWithFrame:frame]) {
        self.backgroundColor = [UIColor whiteColor];
        self.userInteractionEnabled = YES;
        self.multipleTouchEnabled = YES;
        self.contentMode = UIViewContentModeRedraw;
        self.clearsContextBeforeDrawing = YES;
        self.opaque = YES;
        self.drawn = NO;
        self.tilted = NO;
    }
    return self;
}

- (void)drawRect:(CGRect)rect
{
    
    self.aDial = [[SundialData alloc] init];
    
    // time functions
    self.theTime = [[NSString alloc] initWithString:[self.aDial getDateTimeFormat:@"timesun"]]; // only gets hours to display hand on sundial
    
    
    // defines a default square for making a circle with radius equal to it's sides, in the center of screen
    if (self.drawn == NO) { // first time view is drawn
        rect = CGRectMake(self.bounds.size.width * 0.25, self.bounds.size.height * 0.25, self.bounds.size.width * 0.5, self.bounds.size.width * 0.5);
    } else {
        rect = self.aRect;
    }
    
    self.theOrigin = rect.origin;
    if (self.tilted == NO) {
        self.aSize = CGSizeMake(rect.size.width, rect.size.height);
    }
    
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetLineWidth(context, 4.0);
    CGContextSetStrokeColorWithColor(context, 
                                     [UIColor redColor].CGColor);
    
    // draw sundial
    CGContextAddEllipseInRect(context, rect);
    CGContextStrokePath(context);
    
    // draw pole
    CGContextSetLineWidth(context, 2.0);
    CGContextSetStrokeColorWithColor(context, [UIColor greenColor].CGColor);
    CGContextAddEllipseInRect(context, CGRectMake(self.theOrigin.x + (self.aSize.width * 0.5), self.theOrigin.y + (self.aSize.height * 0.5), 4.0, 4.0));
    CGContextStrokePath(context);
                        
    // prepare to draw hour hand - "shadow"
    CGContextSetLineWidth(context, 1.0);
    CGContextSetStrokeColorWithColor(context, [UIColor blackColor].CGColor);

    // define starting point of a line for drawing hour hands of a clock using 12-equally spaced points along circle diameter
    
    if ([self.theTime isEqualToString:@"12"]) {
        // 12-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.5), self.theOrigin.y);
    }
    
    else if ([self.theTime isEqualToString:@"13"]) {
        // 1-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.67), self.theOrigin.y + (self.aSize.height * 0.17));
    }
    
    else if ([self.theTime isEqualToString:@"14"]) {
        // 2-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.84), self.theOrigin.y + (self.aSize.height * 0.34));
    }
    
    else if ([self.theTime isEqualToString:@"15"]) {
        // 3-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 1.0), self.theOrigin.y + (self.aSize.height * 0.5));
    }
    
    else if ([self.theTime isEqualToString:@"16"]) {
        // 4-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.84), self.theOrigin.y + (self.aSize.height * 0.67));
    }
    
    else if ([self.theTime isEqualToString:@"17"]) {
        // 5-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.67), self.theOrigin.y + (self.aSize.height * 0.84));
    }
    
    else if ([self.theTime isEqualToString:@"06"] || [self.theTime isEqualToString:@"18"]) {
        // 6-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.5), self.theOrigin.y + (self.aSize.height * 1.0));
    }
    
    else if ([self.theTime isEqualToString:@"07"] || [self.theTime isEqualToString:@"19"]) {
        // 7-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.34), self.theOrigin.y + (self.aSize.height * 0.83));    
    }
    
    else if ([self.theTime isEqualToString:@"08"] || [self.theTime isEqualToString:@"20"]) {
        // 8-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.17), self.theOrigin.y + (self.aSize.height * 0.66));
    }
    
    else if ([self.theTime isEqualToString:@"09"]) {
        // 9-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x, self.theOrigin.y + (self.aSize.height * 0.5));
    }
    
    else if ([self.theTime isEqualToString:@"10"]) {
        // 10-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.17), self.theOrigin.y + (self.aSize.height * 0.34));
    }
    
    else if ([self.theTime isEqualToString:@"11"]) {
        // 11-o-clock
        CGContextMoveToPoint(context, self.theOrigin.x + (self.aSize.width * 0.34), self.theOrigin.y + (self.aSize.height * 0.17));
    }
    
    else {
        NSLog(@"It's either too early or too late for the sun to make any shadows on this dial!");
    }
    
    if (!([self.theTime isEqualToString:@"21"] || [self.theTime isEqualToString:@"22"] || [self.theTime isEqualToString:@"23"] || [self.theTime isEqualToString:@"00"] || [self.theTime isEqualToString:@"01"] || [self.theTime isEqualToString:@"02"] || [self.theTime isEqualToString:@"03"] || [self.theTime isEqualToString:@"04"] || [self.theTime isEqualToString:@"05"])) {
        // end-point for hour hand
        CGContextAddLineToPoint(context, self.theOrigin.x + self.aSize.width * 0.5, self.theOrigin.y + self.aSize.height * 0.5);
        
        CGContextStrokePath(context);
        self.drawn = YES;   // view is drawn once after first time, now rect will not be default, it will be received from heading info
    }
    
    self.aRect = rect;
    
    [self getGyro:self.aDial];
}

- (void)getGyro:(SundialData *)theDial
{
    theDial.motionManager = [[CMMotionManager alloc] init];
    NSOperationQueue *aQueue = [[NSOperationQueue alloc] init];
    if (theDial.motionManager.gyroAvailable) {
        theDial.motionManager.gyroUpdateInterval = 1.0/10.0;
        [theDial.motionManager startGyroUpdatesToQueue:aQueue withHandler:
         ^(CMGyroData *gyroData, NSError *error) {
             if (error) {
                 [theDial.motionManager stopGyroUpdates];
             } else {
                 theDial.gyroX = gyroData.rotationRate.x; 
                 theDial.gyroY = gyroData.rotationRate.y; 
                 theDial.gyroZ = gyroData.rotationRate.z;
             }
             // performs method which varies length of shadow according to tilt
          //   [self performSelectorOnMainThread:@selector(checkTilt:) withObject:theDial waitUntilDone:YES];
         }];
    }
}

 // changing reference point to reduce length of "shadow" hour hand
- (void)checkTilt:(SundialData *)myData
{
    if (myData.gyroX > 0) {
        self.tilted = YES;
        self.aSize = CGSizeMake(self.aSize.width - myData.gyroX, self.aSize.height);
        if (myData.gyroY > 0) {
            self.aSize = CGSizeMake(self.aSize.width, self.aSize.height + myData.gyroY);
        } else if (myData.gyroY < 0) {
            self.aSize = CGSizeMake(self.aSize.width, self.aSize.height - myData.gyroY);
        }
    } else if (myData.gyroX < 0) {
        self.tilted = YES;
        self.aSize = CGSizeMake(self.aSize.width + myData.gyroX, self.aSize.height);
        if (myData.gyroY > 0) {
            self.aSize = CGSizeMake(self.aSize.width, self.aSize.height + myData.gyroY);
        } else if (myData.gyroY < 0) {
            self.aSize = CGSizeMake(self.aSize.width, self.aSize.height - myData.gyroY);
        }
    }
    [self setNeedsDisplayInRect:self.aRect];
}

@end
