//
//  ViewController.m
//  SunDial
//
//  Created by SJSU Students on 4/18/12.
//  Copyright (c) 2012 SJSU Students. All rights reserved.
//

#import "ViewController.h"

#import "SundialView.h"

@interface ViewController ()

// private properties and methods

@property (strong, nonatomic) NSString *theDate;
@property (strong, nonatomic) NSString *theTime;
@property CGRect myRect;

- (void)handleGesture:(UIGestureRecognizer *)gestureRecognizer;
- (void)updateInfoView;
- (void)drawSundial;
- (void)getTiltInfo;
- (void)getHeadingOrientationAndMakeRect;

@end

@implementation ViewController
@synthesize sundial = _sundial;
@synthesize info = _info;
@synthesize dateLabel = _dateLabel;
@synthesize timeLabel = _timeLabel;
@synthesize trueNorthLabel = _trueNorthLabel;
@synthesize latitudeLabel = _latitudeLabel;
@synthesize longitudeLabel = _longitudeLabel;
@synthesize tiltLabel = _tiltLabel;
@synthesize swipeRight = _swipeRight;
@synthesize doubleTap = _doubleTap;
@synthesize mySundial = _mySundial;
@synthesize theDate = _theDate;
@synthesize theTime = _theTime;
@synthesize myRect = _myRect;
@synthesize theTitle = _theTitle;

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //sundial is primary view
    self.sundial = [[SundialView alloc] initWithFrame:self.view.bounds];
    [self.view addSubview:self.sundial];
    
    [self drawSundial];
    
    [self.sundial addSubview:self.theTitle];
    
    self.info = [[UIView alloc] init];
    [self.info removeFromSuperview];
    
    //adding touch recognizers
    self.swipeRight = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleGesture:)];
    self.swipeRight.direction = UISwipeGestureRecognizerDirectionRight;
    [self.view addGestureRecognizer:self.swipeRight];
    
    self.doubleTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleGesture:)];
    self.doubleTap.numberOfTapsRequired = 2;
    [self.view addGestureRecognizer:self.doubleTap];
    
    // creating sundial model for my controller
    self.mySundial = [[SundialData alloc] init];
    
    // date and time functions
    self.theDate = [[NSString alloc] initWithString:[self.mySundial getDateTimeFormat:@"dateinfo"]];
    self.theTime = [[NSString alloc] initWithString:[self.mySundial getDateTimeFormat:@"timeinfo"]]; // only gets hours to display hand on sundial
    
    // location functions
    self.mySundial.locationManager = [[CLLocationManager alloc] init];
    self.mySundial.locationManager.desiredAccuracy = kCLLocationAccuracyBest;
    self.mySundial.locationManager.distanceFilter = 1000;
    self.mySundial.locationManager.delegate = self;
    [self.mySundial.locationManager startUpdatingLocation];
    if ([CLLocationManager headingAvailable]) {
        [self.mySundial.locationManager startUpdatingHeading];
        self.mySundial.headingAvailbl = @"Compass value: ";
    } else {
        self.mySundial.myTrueNorth = [NSString stringWithFormat:@"0"];
        self.mySundial.headingAvailbl = @"No Compass! Using default=";
        // no heading assumes north at default value
        
        // defines a default square for making a circle with radius equal to it's sides, in the center of screen
        self.myRect = CGRectMake(self.sundial.bounds.size.width * 0.25, self.sundial.bounds.size.height * 0.25, self.sundial.bounds.size.width * 0.5, self.sundial.bounds.size.width * 0.5);
    }
}

- (void)viewDidUnload
{
    self.sundial = nil;
    self.info = nil;
    self.dateLabel = nil;
    self.timeLabel = nil;
    self.trueNorthLabel = nil;
    self.latitudeLabel = nil;
    self.longitudeLabel = nil;
    self.tiltLabel = nil;
    self.swipeRight = nil;
    self.doubleTap = nil;
    self.mySundial = nil;
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Location manager delegate methods

- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation
{
    self.mySundial.myLatitude = [NSString stringWithFormat:@"%g\u00B0",newLocation.coordinate.latitude];
    self.mySundial.myLongitude = [NSString stringWithFormat:@"%g\u00B0",newLocation.coordinate.longitude];
}

- (void) locationManager:(CLLocationManager *)manager didUpdateHeading:(CLHeading *)newHeading
{
    self.mySundial.myTrueNorth = [NSString stringWithFormat:@"%.0f",newHeading.trueHeading];
    [self getHeadingOrientationAndMakeRect];
}


- (void)locationManager:(CLLocationManager *)manager
       didFailWithError:(NSError *)error {
    NSString *errorType = (error.code == kCLErrorDenied) ?
    @"Access Denied" : @"Unknown Error";
    UIAlertView *alert = [[UIAlertView alloc]
                          initWithTitle:@"Error getting Location"
                          message:errorType
                          delegate:nil
                          cancelButtonTitle:@"Okay"
                          otherButtonTitles:nil];
    [alert show];
}

#pragma mark - Methods to draw views

// writes info to labels on view
- (void)updateInfoView
{
    self.dateLabel.text = self.theDate;
    self.timeLabel.text = self.theTime;
    
    self.trueNorthLabel.text = [NSString stringWithFormat:@"%@%@",self.mySundial.headingAvailbl, self.mySundial.myTrueNorth];
    
    self.latitudeLabel.text = self.mySundial.myLatitude;
    self.longitudeLabel.text = self.mySundial.myLongitude; 
    
    [self getTiltInfo];
}

// draws sundial according to info received from date, time and location
- (void)drawSundial
{
    [self.sundial setNeedsDisplayInRect:self.myRect];
}

#pragma mark - Touch action methods

// performs action as per received gesture
- (void)handleGesture:(UIGestureRecognizer *)gestureRecognizer
{
    if (gestureRecognizer == self.swipeRight && self.view.subviews.lastObject == self.sundial) {    // gesture in sundial view
        [self.sundial removeFromSuperview];
        [self.view addSubview:self.info];
        [self updateInfoView];
    } else if (gestureRecognizer == self.doubleTap && self.view.subviews.lastObject == self.info) { // gesture in info view
        [self.info removeFromSuperview];
        [self.view addSubview:self.sundial];
        [self drawSundial];
    }
}

#pragma mark - Motion manager methods

// gets gyro info and writes to label text
- (void)getTiltInfo
{
    self.mySundial.motionManager = [[CMMotionManager alloc] init];
    NSOperationQueue *queue = [[NSOperationQueue alloc] init];
    if (self.mySundial.motionManager.gyroAvailable) {
        self.mySundial.motionManager.gyroUpdateInterval = 1.0/10.0;
        [self.mySundial.motionManager startGyroUpdatesToQueue:queue withHandler:
         ^(CMGyroData *gyroData, NSError *error) {
             NSString *tiltLabelText;
             if (error) {
                 [self.mySundial.motionManager stopGyroUpdates];
                 tiltLabelText = [NSString stringWithFormat:
                                        @"Gyro error: %@", error];
             } else {
                 tiltLabelText = [NSString stringWithFormat:
                                  @"x: %+.2f y: %+.2f z: %+.2f", 
                                  gyroData.rotationRate.x, 
                                  gyroData.rotationRate.y, 
                                  gyroData.rotationRate.z];
             }
             [self.tiltLabel performSelectorOnMainThread:@selector(setText:) 
                                              withObject:tiltLabelText 
                                           waitUntilDone:NO];
         }];
    } else {
        self.tiltLabel.text = @"No Gyro!";
    }
}

// gets heading orientation and makes a rect for drawing into with appropriate origin
- (void)getHeadingOrientationAndMakeRect
{
    CGPoint theOriginAt0 = self.sundial.theOrigin;
    CGPoint theOriginat90 = CGPointApplyAffineTransform(theOriginAt0, CGAffineTransformMakeRotation(M_PI / 2));
    CGPoint theOriginAt180 = CGPointApplyAffineTransform(theOriginAt0, CGAffineTransformMakeRotation(M_PI));
    CGPoint theOriginAt270 = CGPointApplyAffineTransform(theOriginAt0, CGAffineTransformMakeRotation((3 * M_PI) / 2));
    
    switch (self.mySundial.locationManager.headingOrientation) {
        case 1: // portrait, north heading = top
            
            // origin at top-left of default sqaure, this is default square
            self.sundial.theOrigin = theOriginAt0;
            
            self.myRect = CGRectMake(self.sundial.theOrigin.x, self.sundial.theOrigin.y, self.sundial.bounds.size.width * 0.5, self.sundial.bounds.size.width * 0.5);
            break;
            
        case 2: // portrait upside down, north heading = bottom
            
            // origin at bottom-right of default square
            self.sundial.theOrigin = theOriginAt180;
            
            self.myRect = CGRectMake(self.sundial.theOrigin.x, self.sundial.theOrigin.y, self.sundial.bounds.size.width * 0.5, self.sundial.bounds.size.width * 0.5);
            break;
            
        case 3: // landscape left, north heading = right
            
            // origin at top-right of default square
            self.sundial.theOrigin = theOriginat90;
            
            self.myRect = CGRectMake(self.sundial.theOrigin.x, self.sundial.theOrigin.y, self.sundial.bounds.size.width * 0.5, self.sundial.bounds.size.width * 0.5);
            break;
            
        case 4: // landscape right, north heading = left
            
            // origin at bottom-left of default square
            self.sundial.theOrigin = theOriginAt270;
            
            self.myRect = CGRectMake(self.sundial.theOrigin.x, self.sundial.theOrigin.y, self.sundial.bounds.size.width * 0.5, self.sundial.bounds.size.width * 0.5);
            break;
            
        default:    
            // default square
            self.myRect = CGRectMake(self.sundial.theOrigin.x, self.sundial.theOrigin.y, self.sundial.bounds.size.width * 0.5, self.sundial.bounds.size.width * 0.5);
            break;
    }
    [self drawSundial];
}

@end
